from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class Pagination(PageNumberPagination):
    page_size_query_param = 'page_size'
    max_page_size = 50

    def get_paginated_response(self, data):
        return Response({
            'page': self.page.number,
            'pageSize': self.page.paginator.per_page,
            'count': self.page.end_index() - self.page.start_index() + 1,
            'total': self.page.paginator.count,
            'pagesCount': self.page.paginator.num_pages,
            'data': data,
        })


class LargeResultsSetPagination(Pagination):
    page_size = 50


class StandardResultsSetPagination(Pagination):
    page_size = 20


class SmallResultsSetPagination(Pagination):
    page_size = 10


class MiniResultsSetPagination(Pagination):
    page_size = 5



from .dict_util import get_by_dotted as dict_get_dotted
from .date import EDate, EDateTime, to_edatetime, to_edate, generate_months
from .data import EData
from .color import generate_random_color, hex_to_rgb, rgb_to_hex, darken_color

__all__ = [
    'dict_get_dotted',
    'EData',
    'EDate',
    'EDateTime',
    'to_edatetime',
    'to_edate',
    'generate_months',
    'generate_random_color',
    'hex_to_rgb',
    'rgb_to_hex',
    'darken_color',
]



from rest_framework.permissions import IsAuthenticated

from energy_base.utils.microservices import is_internal_microservice_request


class BaseRolePermissions(IsAuthenticated):
    required_role = None

    def has_permission(self, request, view):
        if not super().has_permission(request, view):
            return False

        if is_internal_microservice_request(request):
            return True

        if hasattr(request.user, 'roles'):
            return self.required_role in request.user.roles

        return False


import importlib
from urllib.parse import urlparse

import inflect
from django.conf import settings
from django.db.models import Model, CharField
from drf_spectacular.utils import extend_schema, extend_schema_serializer
from rest_framework import filters
from rest_framework import viewsets, serializers

from ..api.views import TranslatedListView

serializer_classes = {}


def find_model(model_name: str):
    for model_class in get_model_classes():
        if get_model_class_name(model_class).lower() == model_name.lower():
            return model_class


def get_api_meta_property(model: Model, property_name: str):
    if hasattr(model, 'ApiMeta'):
        if hasattr(model.ApiMeta, property_name):
            return getattr(model.ApiMeta, property_name)


def get_model_class_name(model: Model):
    return model._meta.object_name


def get_model_fields_list(model: Model):
    return model._meta.get_fields()


def get_view_set_name(model: Model):
    return get_model_class_name(model) + 'ViewSet'


def get_basename(model: Model):
    return 'admin-' + get_plural_form(get_model_class_name(model).lower())


def camel_to_snake(name: str):
    return ''.join(map(lambda c: '-' + c.lower() if c.isupper() else c, name)).removeprefix('-')


def get_url_prefix(model: Model):
    return 'admin/' + get_plural_form(camel_to_snake(get_model_class_name(model))).replace(' ', '-')


def get_plural_form(word: str):
    p = inflect.engine()
    return p.plural(word)


def get_serializer_name(model: Model):
    return get_model_class_name(model) + 'Serializer'


def get_ordering(model: Model):
    if model._meta.ordering:
        return model._meta.ordering
    if hasattr(model, 'created_at'):
        return ('-created_at',)
    if hasattr(model, 'id'):
        return ('id',)
    return ()


def get_model_classes():
    model_classes = []
    for model_import_path in settings.REFERENCE_API_MODELS:
        module_name, class_name = model_import_path.rsplit('.', 1)
        module = importlib.import_module(module_name)
        model_classes.append(getattr(module, class_name))

    return model_classes


def create_view_set(model: Model):
    fields_list = get_model_fields_list(model)

    @extend_schema(tags=[get_url_prefix(model)])
    class ViewSet(viewsets.ModelViewSet, TranslatedListView):
        http_method_names = get_api_meta_property(model, 'http_method_names') or ['get', 'patch']
        manager = model.objects.order_by(*get_ordering(model))
        filter_backends = [filters.SearchFilter]
        search_fields = (get_api_meta_property(model, 'search_fields') or
                         [field.name for field in fields_list if isinstance(field, CharField)])

        def get_serializer_class(self):
            if self.action == 'create':
                return create_serializer_for_create(model)

            if self.action == 'partial_update':
                return create_serializer_for_partial_update(model)

            return create_serializer(model)

    return ViewSet


def create_serializer_for_create(model: Model):
    @extend_schema_serializer(component_name='Admin' + get_serializer_name(model))
    class Serializer(serializers.ModelSerializer):
        class Meta:
            fields = '__all__'

    Serializer.Meta.model = model
    return Serializer


def create_serializer_for_partial_update(model: Model):
    fields_list = get_model_fields_list(model)

    @extend_schema_serializer(component_name='Admin' + get_serializer_name(model))
    class Serializer(serializers.ModelSerializer):
        class Meta:
            fields = '__all__'
            read_only_fields = list(
                {
                    'created_at',
                    'updated_at',
                    'deleted_at',
                    'created_by',
                    'updated_by',
                    'deleted_by',
                    'code'
                } & set([field.name for field in fields_list])
            )

    Serializer.Meta.model = model
    return Serializer


def create_serializer(model: Model, depth=0):
    if get_model_class_name(model) in serializer_classes:
        return serializer_classes[get_model_class_name(model)]

    fields_list = get_model_fields_list(model)

    class ImageNameField(serializers.ImageField):
        def to_representation(self, value):
            if not value:
                return None
            parsed_url = urlparse(value.url)
            return f'{parsed_url.path}?{parsed_url.query}'

    @extend_schema_serializer(component_name='Admin' + get_serializer_name(model))
    class Serializer(serializers.ModelSerializer):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            nonlocal model, fields_list
            for field in fields_list:
                if field.__class__.__name__.endswith('ImageField'):
                    self.fields[field.name] = ImageNameField()

                if field.__class__.__name__.endswith('ForeignKey'):
                    model_name = field.name
                    _model = find_model(model_name)
                    if _model is not None and depth == 0:
                        self.fields[model_name] = create_serializer(_model, depth + 1)()

        class Meta:
            fields = '__all__'
            read_only_fields = list(
                {
                    'created_at',
                    'updated_at',
                    'deleted_at',
                    'created_by',
                    'updated_by',
                    'deleted_by',
                    'code'
                } & set([field.name for field in fields_list])
            )

    Serializer.Meta.model = model
    serializer_classes[get_model_class_name(model)] = Serializer
    return Serializer



from django.conf import settings
from django.utils.module_loading import import_string
from rest_framework.permissions import IsAuthenticated

from .admin_user import IsAdminUser
from .ges import IsGesUser
from .het import IsHetUser
from .hududgaz_liqufiedgas import IsHududGazLiqufiedGasUser
from .hududgaz_naturalgaz import IsHududGazNaturalgazUser
from .ies import IsIesUser
from .ies_file import IsIesFileUser
from .liquidhydrocarbons import IsLiquidHydrocarbonsUser
from .met import IsMetUser
from .national_dispatch_center import IsNationalDispatchUser
from .neftegaz import IsNeftegazUser
from .neftegaz_file import IsNeftegazFileUser
from .neftegaz_naturalgaz import IsNeftegazNaturalgazUser
from .neftegaz_oil_well import IsOilWellUser
from .superuser import IsSuperUser
from .uzbekkomir import IsUzbekkomirUser
from .uzgastrade import IsUzGasTradeUser
from .uztransgaz import IsUzTransGazUser
from .uztransgaz_file import IsUzTransGazFileUser
from .smart_integrity import IsSmartIntegrityUser

roles = settings.REST_FRAMEWORK.get('ROLE_PERMISSION_CLASSES', [])

role_classes = []

for role in roles:
    role_classes.append(import_string(role))

if role_classes:
    DEFAULT_ROLE_PERMISSIONS = role_classes[0]
else:
    DEFAULT_ROLE_PERMISSIONS = IsAuthenticated

for role_class in role_classes[1:]:
    DEFAULT_ROLE_PERMISSIONS = DEFAULT_ROLE_PERMISSIONS | role_class

__all__ = [
    'IsSuperUser', 'IsAdminUser', 'IsGesUser', 'IsHetUser', 'IsHududGazLiqufiedGasUser',
    'IsIesUser', 'IsLiquidHydrocarbonsUser', 'IsHududGazNaturalgazUser', 'IsMetUser', 'IsOilWellUser',
    'IsNeftegazNaturalgazUser', 'IsUzGasTradeUser', 'IsUzTransGazUser', 'IsUzbekkomirUser', 'IsIesFileUser',
    'IsUzTransGazFileUser', 'IsNeftegazFileUser', 'IsNationalDispatchUser', 'IsNeftegazUser', 'IsSmartIntegrityUser',
    'DEFAULT_ROLE_PERMISSIONS'
]



import importlib

from django.conf import settings
from django.core.management.base import BaseCommand
from django_q.models import Schedule


class Command(BaseCommand):
    help = "Migrate tasks"

    def add_arguments(self, parser):
        parser.add_argument('--app', type=str, help="An optional argument", required=True)

    def handle(self, *args, **options):
        try:
            tasks = importlib.import_module(f"apps.{options['app']}.tasks")
            tasks = tasks.tasks
            for task in tasks:
                scheduler = Schedule.objects.filter(name=task['name']).first()
                if not scheduler:
                    scheduler = Schedule()
                else:
                    task.pop('next_run')
                for k, v in task.items():
                    setattr(scheduler, k, v)
                scheduler.save()
            self.stdout.write(
                self.style.SUCCESS('Successfully migrated tasks')
            )
        except ModuleNotFoundError:
            self.stdout.write(self.style.ERROR("Tasks module not found"))



from django.db.models import Manager
from rest_framework import generics

from .base import BaseAPIView


class TranslatedListView(generics.ListAPIView, BaseAPIView):
    manager: Manager

    def get_queryset(self):
        self.update_lang()
        return self.manager.all()



from calendar import monthrange
from datetime import date, datetime, timedelta

DEFAULT_DATE_FORMAT = '%Y-%m-%d'
DEFAULT_DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'


def to_edate(date: date | datetime):
    return EDate(date.year, date.month, date.day)


def to_edatetime(dt: datetime):
    return EDateTime(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)


class EDate(date):
    def datetime(self):
        return datetime(self.year, self.month, self.day)

    def tomorrow(self):
        return to_edate(self.datetime() + timedelta(days=1))

    def yesterday(self):
        return to_edate(self.datetime() - timedelta(days=1))

    def first_day_of_month(self):
        return self.replace(day=1)

    def first_day_of_year(self):
        return self.replace(day=1, month=1)

    def last_day_of_month(self):
        _, ndays = monthrange(self.year, self.month)
        return self.replace(day=ndays)

    def last_day_of_year(self):
        return self.replace(day=31, month=12)

    def first_day_of_last_month(self):
        return self.replace(day=1).yesterday().replace(day=1)

    def first_day_of_last_year(self):
        return self.replace(day=1, month=1, year=self.year - 1)

    def last_day_of_last_month(self):
        return self.replace(day=1).yesterday()

    def last_day_of_last_year(self):
        return self.replace(day=31, month=12, year=self.year - 1)

    def strftime(self, __format=DEFAULT_DATE_FORMAT):
        return super().strftime(__format)

    @staticmethod
    def strptime(__date_string: str, __format=DEFAULT_DATE_FORMAT):
        return to_edate(datetime.strptime(__date_string, __format))

    def __str__(self):
        return self.strftime()


class EDateTime(datetime):
    def first_second_of_minute(self):
        return self.replace(second=0, microsecond=0)

    def last_second_of_minute(self):
        return self.replace(second=59)

    def first_second_of_hour(self):
        return self.first_second_of_minute().replace(minute=0)

    def last_second_of_hour(self):
        return self.last_second_of_minute().replace(minute=59)

    def first_second_of_day(self):
        return self.first_second_of_hour().replace(hour=0)

    def last_second_of_day(self):
        return self.last_second_of_hour().replace(hour=23)

    def one_hour_later(self):
        return self + timedelta(hours=1)

    def hour_ago(self):
        return self - timedelta(hours=1)

    def tomorrow(self):
        return self + timedelta(days=1)

    def yesterday(self):
        return self - timedelta(days=1)

    def first_day_of_month(self):
        return self.replace(day=1)

    def last_day_of_month(self):
        _, ndays = monthrange(self.year, self.month)
        return self.replace(day=ndays)

    def first_day_of_year(self):
        return self.replace(day=1, month=1)

    def last_day_of_year(self):
        return self.replace(day=31, month=12)

    def first_day_of_last_month(self):
        return self.replace(day=1).yesterday().replace(day=1)

    def last_day_of_last_month(self):
        return self.replace(day=1).yesterday()

    def first_day_of_last_year(self):
        return self.replace(day=1, month=1, year=self.year - 1)

    def last_day_of_last_year(self):
        return self.replace(day=31, month=12, year=self.year - 1)

    def strftime(self, __format=DEFAULT_DATETIME_FORMAT):
        return super().strftime(__format)

    @staticmethod
    def strptime(__date_string: str, __format=DEFAULT_DATETIME_FORMAT):
        return to_edatetime(datetime.strptime(__date_string, __format))

    def __str__(self):
        return self.strftime()


def generate_months():
    for month_number in range(1, 13):
        yield month_number
